@@ -9,6 +9,10 @@ Plots for computational materials/molecular science
 
 # Main classes
 from .dashboard import Dashboard  # noqa: F401
+from .dashboard import DashboardConnectionError  # noqa: F401
+from .dashboard import DashboardLoginError  # noqa: F401
+from .dashboard import DashboardTimeoutError  # noqa: F401
+from .dashboard import DashboardUnknownError  # noqa: F401
 
 # Handle versioneer
 from ._version import get_versions
